@@ -3,7 +3,7 @@
 
 sh simple_lock l1, l2;
 
-main() {            /* !deadlock! */
+void main( void ) {            /* !deadlock! */
   simple_lock_init( l1 );   simple_lock_init( l2 );
   if (__PROC_NR__ % 2) {
      simple_lockup( &l1 );
