@@ -4,7 +4,7 @@
 #include <io.h>
 
 
-main() 
+void main(void) 
 {
  pr unsigned int t = getct();
  if (__PROC_NR__==0)
