@@ -56,6 +56,11 @@ extern async int async_groupsize( void );   /* same function, but other prototyp
 extern sync int parentgroupsize( void );
 extern sync int _groupsize( unsigned int ); /*new 970901*/
 
+extern int *__TICKETP__;   /* points to my current busTicket counter 990128*/
+extern int *__RANKP__;   /* points to my current busRank counter 990128*/
+#define __NUM_PR__ (*__TICKETP__)
+#define __RANK__ (*__RANKP__)
+
 
 /* =================  L O C K S :  ======================================== */
 
@@ -222,6 +227,14 @@ extern int rw_unlock( rw_lock *, int, int );
 
 /* ==============    J O I N    ---    the bus concept    950918 ============= */
 
+/* Old implementation of the join construct as a macro
+ * is now obsolete. Use the join statement supported by
+ * the compiler, it has more features and is easier to use
+ * than this macro.
+ * The macro is classified as deprecated in version 1.8 and
+ * will no longer be supported in future versions.
+ */
+
 /* global variables for the bus concept: */
 
 #define _MAXJOINS 8         /* maximal number of joins in my user program */
@@ -256,7 +269,7 @@ extern sh int _ticket[_MAXJOINS];  /* semaphore to control entry of passengers *
     _busywait the statement to be executed by procs that missed the bus or spring off
  */
 
-#define join(_joinID,_delaystmt,_throwoutcond,_SMsize,_retry,_body,_busywait) { \
+#define JOIN(_joinID,_delaystmt,_throwoutcond,_SMsize,_retry,_body,_busywait) { \
   pr int _old_dollar = $;  \
   pr int _redo = 0;  \
   do { \
@@ -290,8 +303,10 @@ extern sh int _ticket[_MAXJOINS];  /* semaphore to control entry of passengers *
             /*write(1,"gone!\n",6);*/ \
             _gone[_joinID] = 1; \
             asm("nop\nnop\nnop\nnop\nnop\nnop\nnop"); \
+            /* now __ticket is stable: */ \
             asm("gethi\t __ticket,r30"); /* r30 <- _ticket[_joinID]:*/ \
             asm("add\t r30,__ticket & 0x1fff,r30"); \
+            asm("getlo\t " str(_joinID) ",r31"); \
             force_modulo_0; \
             asm("ldg\t r30,r31,r30"); \
             asm("nop"); \
@@ -309,13 +324,13 @@ extern sh int _ticket[_MAXJOINS];  /* semaphore to control entry of passengers *
             asm("add\t spp,-1,gpp"); \
             asm("pshgc\t spp"); /* @ = 0 */ \
             asm("pshg\t r31,spp"); /* new $ is my ticket number */ \
-            asm("getlo\t " str(_joinID) ",r31"); \
             while (!_gone[_joinID]) /*wait*/; \
             /*pprintf(" see gone=1, ticket=%d, $=%d\n",_ticket[_joinID], $);*/ \
             asm("nop\nnop\nnop\nnop\nnop\nnop\nnop"); \
             /* now the value in _SM[joinID] is valid: */ \
             asm("gethi\t __SM,gps"); /* compute new gps as _SM[_joinID]: */ \
             asm("add\t gps,__SM & 0x1fff,gps"); \
+            asm("getlo\t " str(_joinID) ",r31"); \
             force_modulo_0; \
             asm("ldg\t gps,r31,gps"); \
             asm("nop"); \
@@ -357,6 +372,7 @@ extern sh int _ticket[_MAXJOINS];  /* semaphore to control entry of passengers *
             asm("popg\t spp,sps"); \
             asm("popg\t spp,eps"); \
             asm("popg\t spp,gps"); \
+            if ($==0) shfree( _SM[_joinID] ); \
             $ = _old_dollar;    /* restore old $ value */ \
       } \
     } \
@@ -369,7 +385,7 @@ extern sh int _ticket[_MAXJOINS];  /* semaphore to control entry of passengers *
   } \
   while (_redo); \
 } \
-/* end of join macro. */
-
+/* end of JOIN macro. */
 
 #endif
+
