@@ -56,6 +56,22 @@ extern void prF( float x );
 extern void prstat( void );
  /* prints status message containing Processor ID, current $ and @ values */
 
+extern unsigned int _barrcnt;    /*990119, defined in forklib2.asm*/
+extern sh unsigned int _shldgcnt;   /*        printed by printAccStat*/
+extern sh unsigned int _shstgcnt;
+extern sh unsigned int _mpaddcnt;
+extern sh unsigned int _mpmaxcnt;
+extern sh unsigned int _mpandcnt;
+extern sh unsigned int _mporcnt;
+extern unsigned int _myshldgcnt;   /*        printed by printAccStat*/
+extern unsigned int _myshstgcnt;
+extern unsigned int _mympaddcnt;
+extern unsigned int _mympmaxcnt;
+extern unsigned int _mympandcnt;
+extern unsigned int _mymporcnt;
+extern void printAccStat( void );   /*990119*/
+ /* prints shared memory access statistics for this processor */
+
 extern sh char __CR[];
 
 #define prIln(n) {prI(n,0);write(1,"\n",1);}
