@@ -94,6 +94,25 @@ void prstat( void ) {
 }
 
 
+void printAccStat( void )         /* 990119*/
+{
+  write(1,"AccStat for P", 13); prI(__PROC_NR__,0); write(1,": passed ",9);
+  prI(_barrcnt,0); write(1," barriers,\n                ",27);
+  prI(_myshldgcnt,0); write(1," of ",4);
+  prI(_shldgcnt,0); write(1," sh-loads,\t",11);
+  prI(_myshstgcnt,0); write(1," of ",4);
+  prI(_shstgcnt,0); write(1," sh-stores,\n                ",28);
+  prI(_mympaddcnt,0); write(1," of ",4);
+  prI(_mpaddcnt,0); write(1," mp/syncadds,\t",14);
+  prI(_mympmaxcnt,0); write(1," of ",4);
+  prI(_mpmaxcnt,0); write(1," mp/syncmaxs,\n                ",30);
+  prI(_mympandcnt,0); write(1," of ",4);
+  prI(_mpandcnt,0); write(1," mp/syncands,\t",14);
+  prI(_mymporcnt,0); write(1," of ",4);
+  prI(_mporcnt,0); write(1," mp/syncors\n",12);
+}
+
+
 
 /* ============ scanf()   CWK  ! THIS IS A HACK !   3.11.1995 ====== */
 
