@@ -2,6 +2,7 @@
 
 #include <math.h>
 #include <assert.h>   /* only for test purposes! throw out if possible */
+#include <io.h>
 
 pr int EDOM = 0;  /* Flag for Domain Error */
 pr int ERANGE = 0; /* Flag for Range Error */
@@ -156,3 +157,58 @@ float log10 ( float x )
 {
  return ln(x) * _log_ten_e;
 }
+
+
+float blubberipow( float x, int n )
+{
+ float po = x;
+ if (n==0) return _one;
+ else 
+   for (;n>1; n--)  po *= x;
+ return po;
+}
+
+
+float blubberpow( float x, float y )     /* x^y nach Bronstein */
+{
+ register float z;
+ register float sum = _one;
+ register float bruch = _one;
+ register int i = 0;
+ z = (y * ln( ABS(x) ));
+ do {
+   i++;
+   bruch = bruch * (z / (double)i);
+   sum = sum + bruch; 
+ } while (bruch >= _accuracy && i < 100);
+                        /* Notbremse CWK */
+ return sum; 
+}
+
+
+float pow( float x, float y )
+{
+ int iy = (int)(floor(fabs(y+_accuracy)));
+
+ /* domain error if x is zero and y is negative or zero: */
+ if (x<_accuracy&&-x>_accuracy && y<=(float)0) { EDOM = 1; fperror("pow"); }
+
+ /* if x is zero and y is positive, then the result is zero. */
+ if (ABS(x)<_accuracy && y>0.0) return 0.0;
+
+ /* if x is non-zero and y is zero, then the result is 1.0. */
+ if (ABS(x)>=_accuracy && ABS(y)<_accuracy) return 1.0;
+
+ /* if x is negative and y is negative: */
+ if (x<(float)0 && y<(float)0) { EDOM = 1; fperror("pow"); }
+
+ /* if x is negative and y is an integer: */
+ if (x<0.0 && fabs(y - iy) < _accuracy)
+    return blubberipow(x, (int)(y));
+
+ /* domain error if x is negative and y is not an integer: */
+ if (x<(float)0) 
+    { EDOM = 1; fperror("pow"); }
+
+ return blubberpow( x, y );
+}
