@@ -73,10 +73,14 @@ static void checkref(p, cl) Symbol p; Generic cl; {
 void compound(loop, swp, lev, bframes, cframes, rframes) struct swtch *swp; { /*CWK*/
    Symbol p;
    Code cp;
+   Code hilf;  /*970903*/
    int i, j, nregs;
+   extern shspace( int );
 
    walk(0, 0, 0);
    cp = code(Blockbeg);
+   shspace( 0 );   /*970902*/
+   hilf = codelist;  /*970903*/
    enterscope();
    cp->u.block.bnumber = ++bnumber;
    cp->u.block.level = level;
@@ -98,6 +102,24 @@ void compound(loop, swp, lev, bframes, cframes, rframes) struct swtch *swp; { /*
    nregs = length(registers);
    cp->u.block.locals = (Symbol *)ltoa(registers, (Generic *)talloc((nregs + length(autos) + 1)*sizeof(Symbol)));
    ltoa(autos, (Generic *)&cp->u.block.locals[nregs]);
+
+   {
+   Symbol *myp;
+   int k;
+   myp = cp->u.block.locals;
+   for (k=0 ; *myp; myp++)
+     if (1
+/*        ((*myp)->ref > 0 
+         || (*myp)->initialized || glevel )
+           auskommentiert: mit klappt's leider nicht, es wird
+           zuwenig Platz allokiert!    970908 */
+       && (*myp)->shared )
+        k+=(*myp)->type->size;   /*980203 statt ++ */
+   if (k>0) 
+      hilf->par = k;  /* shspace( k );   970902 backpatchen*/
+   walk(0,0,0);
+   }
+
    while (kind[t] == IF || kind[t] == ID)
       statement(loop, swp, lev, bframes, cframes, rframes);         /*CWK*/
    walk(0, 0, 0);
@@ -137,12 +159,12 @@ static Symbol dclglobal(sclass, styp, id, ty, pos, sy) char *id; Type ty; Coordi
             p->name, &p->src);
    } else if (sclass == STATIC && !isfunc(ty) && ty->size == 0 && t != '=')
       warning("undefined size for static `%t %s'\n", ty, id);
-    if (p && isfunc(ty) && (p->sync&&(sy==ASYNCF) || !p->sync&&(sy==SYNCF))) /*CWK*/
-       error("inconsistent sync/async declaration of `%s' w.r.t. %w\n",
-             p->name, &p->src);
-    if (p && isfunc(ty) && (ty->u.proto) && *(ty->u.proto)
-          &&(p->sharedparameters != (*(ty->u.proto))->sharedparams))
-       error("inconsistent pr/sh parameter declaration for `%s' w.r.t. %w\n",
+   if (p && isfunc(ty) && (p->sync&&(sy==ASYNCF) || !p->sync&&(sy==SYNCF))) /*CWK*/
+      error("inconsistent sync/async declaration of `%s' w.r.t. %w\n",
+            p->name, &p->src);
+   if (p && isfunc(ty) && (ty->u.proto) && *(ty->u.proto)
+         &&(p->sharedparameters != (*(ty->u.proto))->sharedparams))
+      error("inconsistent pr/sh parameter declaration for `%s' w.r.t. %w\n",
              p->name, &p->src);                                          /*CWK*/
    if (p == 0 || p->scope != GLOBAL)
       p = install(id, &globals, 1);
@@ -192,6 +214,7 @@ static Symbol dclglobal(sclass, styp, id, ty, pos, sy) char *id; Type ty; Coordi
 /* dcllocal - called from decl to declare a local */
 static Symbol dcllocal(sclass, styp, id, ty, pos, sy) char *id; Type ty; Coordinate *pos; { /*CWK*/
    Symbol p, q;
+   extern int shframedepth;  /*970902*/
 
    if (isfunc(ty)) {
       if (sclass && sclass != EXTERN)
@@ -204,7 +227,7 @@ static Symbol dcllocal(sclass, styp, id, ty, pos, sy) char *id; Type ty; Coordin
       sclass = AUTO;
    }
    if ((q = lookup(id, identifiers))
-   && (q->scope >= level || q->scope == PARAM && level == PARAM+1))
+       && (q->scope >= level || q->scope == PARAM && level == PARAM+1))
       if (sclass == EXTERN && q->sclass == EXTERN && eqtype(q->type, ty, 1))
          ty = composite(ty, q->type);
       else
@@ -225,6 +248,8 @@ static Symbol dcllocal(sclass, styp, id, ty, pos, sy) char *id; Type ty; Coordin
    if (p->shared)
       if (!isinsyncmode)
          error("You can't declare a shared object in asynchronous context!\n");
+   assert(shframedepth>=0);
+   p->shframedepth = shframedepth;
    p->src = *pos;
    switch (sclass) {
    case EXTERN: {
@@ -248,9 +273,14 @@ static Symbol dcllocal(sclass, styp, id, ty, pos, sy) char *id; Type ty; Coordin
          if ((q = lookup(id, globals))
               && q->sclass != TYPEDEF && q->sclass != ENUM)
             r = q;
+         if (q && isfunc(ty) && (p->sync != q->sync))    /*CWK*/
+            warning("declaration of synchronicity for `%s' does not match previous declaration at %w\n",
+                    q->name, &q->src);   /*CWK 971121*/
+         if (q && (p->shared != q->shared))    /*CWK*/
+            warning("declaration of sharity for `%s' does not match previous declaration at %w\n",
+                    q->name, &q->src);   /*CWK 971121*/
       }
       if (r && ((r->sclass == AUTO ? EXTERN : r->sclass) != p->sclass
-      || isfunc(ty)&&(p->sync != q->sync) || (p->shared != q->shared)    /*CWK*/
       || !eqtype(r->type, p->type, 1)))
          warning("declaration of `%s' does not match previous declaration at %w\n",
             r->name, &r->src);
@@ -329,7 +359,8 @@ static Symbol dclparam(sclass, styp, id, ty, pos, sy) char *id; Type ty; Coordin
       ty = ptr(ty);
    else if (ty && isarray(ty))
       ty = atop(ty);
-   if (isfunc(ty) && ty->u.proto && *(ty->u.proto)) {  /*CWK*/
+   /* In dieser Bedingung kracht es bei oldstyle Dekl. ohne Abfrage ty:*/
+   if (ty && isfunc(ty) && ty->u.proto && *(ty->u.proto)) {  /*CWK*/
       p->sharedparameters = (*(ty->u.proto))->sharedparams;   /*CWK*/
       p->regparameters = (*(ty->u.proto))->regparams;         /*CWK*/
    }  /*else bleibts bei beiden 0 */
@@ -445,7 +476,7 @@ static Type dclr1(id, lev) char **id; {
    default:
       return ty;
    }
-   while (t == '(' || t == '[')
+   while (t == '(' || t == '[') {
       if (t == '(') {
          t = gettok();
          ty = tnode(FUNCTION, ty);
@@ -471,6 +502,7 @@ static Type dclr1(id, lev) char **id; {
          ty = tnode(ARRAY, ty);
          ty->size = n;
       }
+   }
    return ty;
 }
 
@@ -794,12 +826,13 @@ static void funcdecl(sclass, styp, id, ty, pt, sy) char *id; Type ty; Coordinate
    Code rp;
    Symbol *callee, *caller, p;
    List callees = 0;
+   extern int shframedepth;
 
    regcount = 0;
    if (sy==ASYNCF) { isinsyncmode = 0; }
    else            { isinsyncmode = 1; }
    /*if (styp==SHARED && sy==ASYNCF)
-      error("asynchronous function must not return a shared value\n");       *CWK*/
+      error("asynchronous function must not return a shared value\n");   *CWK*/
    if (isstruct(freturn(ty)) && freturn(ty)->size == 0)
       error("illegal use of incomplete type `%t'\n", freturn(ty));
    callees = 0;
@@ -822,7 +855,7 @@ static void funcdecl(sclass, styp, id, ty, pt, sy) char *id; Type ty; Coordinate
       ty = newstyle(ty, caller, callee);
    }
    else {
-      /*fprint(2,"vor oldstyle(%s):\n", id);  *CWK*/
+      warning("oldstyle function declarations are not supported,\n          sharity of variables will be ignored!\n");  /*CWK*/
       ty = oldstyle(id, ty, caller, callee);
    }
    if (Aflag >= 1 && !hasproto(ty))
@@ -858,6 +891,7 @@ static void funcdecl(sclass, styp, id, ty, pt, sy) char *id; Type ty; Coordinate
    if (ncalled >= 0)
       ncalled = findfunc(cfunc->name, pt.file);
    cfunc->u.f.pt[2] = definept(0)->u.point.src;
+   shframedepth = 0;  /*980206*/
    compound(0, (struct swtch *)0, 0, 0, 0, 0);  /*no frames, no start CWK*/
    for (rp = codelist; rp->kind < Label; rp = rp->prev)
       ;
@@ -988,7 +1022,7 @@ static Type *parameters(lev) {   /*CWK*/
    int akt_parreg = 1;  /*Nummer des ersten Parameter-Registers. CWK*/
 
    /*assert( declaringfunction );   /*CWK*/
-   /*print("declaringfunction = %s\n", declaringfunction);   CWK*/
+   /*fprint(2,"declaringfunction = %s\n", declaringfunction);   /*CWK*/
    enterscope();
    if (kind[t] == CHAR || kind[t] == STATIC
        || kind[t] == SHARED                              /*CWK*/
@@ -1031,12 +1065,12 @@ static Type *parameters(lev) {   /*CWK*/
             akt_parreg = 22222;   /*Varargs: keine weiteren Parregs. CWK*/
          }
          else {
+            /*fprint(2,"parameter id = %s\n", id);   /*CWK*/
             p = dclparam(sclass, styp, id, ty, &src, sy); /*CWK*/
-            /*print("parameter id = %s\n", id);   CWK*/
             p->parreg = 0;  /*CWK: default*/
-            if (styp==SHARED && !is_in_libfnames(declaringfunction)) {
+            if (styp==SHARED && declaringfunction    /*971124*/
+                && !is_in_libfnames(declaringfunction)) {
                    /*zzgl.libf: 950515 CWK; declfn ex. da shared <= Prototyp*/
-               assert(declaringfunction);
                sharedpar = sharedpar | (1<<(n-1)); /*CWK*/
             }
             else     /*CWK*/
@@ -1065,6 +1099,7 @@ static Type *parameters(lev) {   /*CWK*/
             error("expecting an identifier\n");
             break;
          }
+         /* In diesem Aufruf krachte es fuer oldstyle declarations: */
          dclparam(0, 0, token, (Type)0, &src, 0);
          t = gettok();
          if (t != ',')
