@@ -34,6 +34,7 @@ dclproto(static void xxxdostmt,(int, struct swtch *, int, int)); /*CWK*/
 dclproto(static int foldcond,(Tree, Tree));
 dclproto(static void forstmt,(int, struct swtch *, int, int));
 dclproto(static void xxxforstmt,(int, struct swtch *, int, int)); /*CWK*/
+dclproto(static void joinstmt,(int, struct swtch *, int, int)); /*990126*/
 dclproto(static void forkstmt,(int, struct swtch *, int, int, int, int));   /*CWK*/
 dclproto(static void startstmt,(int, struct swtch *, int));   /*CWK*/
 dclproto(static void ifstmt,(int, int, struct swtch *, int, int, int, int));
@@ -332,6 +333,142 @@ static void forstmt(lab, swp, lev, rframes) struct swtch *swp; {
       definelab(lab + 2);
 }
 
+static int joincounter = 0;  /* counts join statements in this source */
+
+/* joinstmt - join ( [expr1] ; [expr2] ; [expr3] ) stmt1 [else stmt2] */
+static void joinstmt(lab, swp, lev, rframes) struct swtch *swp; {
+   Tree e1, e2, e3, b1, b2, b3, b4, b5, b6, b7, b8;
+   int joinid = joincounter++;
+   extern int shframedepth; 
+   int oldshframedepth;
+
+   t = gettok();  /* JOIN */
+   expect('(');
+   definept(0);
+   definelab( lab + 1 );    /* the entry point to the join statment */
+                            /* continue in the else part jumps to here */
+   b1 = tree( JBLOCKa+V, voidtype, 0, 0);
+   /* def vars ticket, gone, SM, SMsize, olddollar */
+   code(Pcode);
+   codelist->u.node = listnodes( b1, 0, 0 );
+   codelist->par = joinid;
+   walk(0,0,0);   /*damit vorangehende stmts geflusht werden*/
+
+   if (kind[t] == ID) {
+      e1 = expr(';');  /* _SMSIZE */
+      code(Gen);
+      codelist->u.node = listnodes( e1, 0, 0 );
+      b2 = tree( JBLOCKb+V, voidtype, e1, 0);    /* store e1 in _SMsize */
+      code(Pcode);
+      codelist->u.node = listnodes( b2, 0, 0 );
+      codelist->par = joinid;  /*damit Sprungziel fuer verpassende Proz. */
+      walk(0,0,0);   /*damit vorangehende stmts geflusht werden*/
+   }
+   else {
+      error("join requires int expression as first parameter (stacksize)");
+      expect(';');
+   }
+   
+   b3 = tree( JBLOCKc+V, voidtype, 0, 0);
+      /* save $, enter group, driver sets up SM */
+   code(Pcode);
+   codelist->u.node = listnodes( b3, 0, 0 );
+   codelist->par = joinid;     /*damit Sprungziel fuer non-Leader in JBd*/
+   walk(0,0,0);   /*damit vorangehende stmts geflusht werden*/
+
+   definelab( lab+4 );
+   if (kind[t] == ID) {   /* _DELAYCOND */
+      e2 = texpr(conditional, ';');
+      walk( e2, 0, lab+4);    /* iterate until true */
+   }
+   else
+      expect(';');       /* no delay. fall through. */
+   
+   b4 = tree( JBLOCKd+V, voidtype, 0, 0);
+   code(Pcode);
+   codelist->u.node = listnodes( b4, 0, 0 );
+   codelist->par = joinid;
+   walk(0,0,0);   /*damit vorangehende stmts geflusht werden*/
+       /* driver finishes sh groupframe, others 
+        * save $, build private group frame, wait for gone,
+        * and enter the bus by setting gps, eps,... to _SM
+        */
+   if (kind[t] == ID) {   /* STAYINSIDECOND */
+      e3 = texpr(conditional, ')');
+      walk( e3, lab+3, 0 );   /* jump to lab+3 (ride) if true */
+      /* ansonsten: Ausstieg und zum Else / DefaultElse springen */
+      b5 = tree( JBLOCKe+V, voidtype, 0, 0); /* leave group frame */
+      code(Pcode);
+      codelist->u.node = listnodes( b5, 0, 0 );
+      walk(0,0,0);   /*damit vorangehende stmts geflusht werden*/
+      branch( lab+5 );
+   }
+   else
+      expect(')');    /* no springoffcond. fall through. */
+
+   definelab( lab+3 );
+   b6 = tree( JBLOCKf+V, voidtype, 0, 0);
+   code(Pcode);
+   codelist->u.node = listnodes( b6, 0, 0 );
+   codelist->par = joinid;
+   /* I ride: synchronize and take off */
+   walk(0,0,0);   /*damit vorangehende stmts geflusht werden*/
+
+   isinsyncmode = 1; isinsideastartbody = 1;
+   oldshframedepth = shframedepth;
+   shframedepth = 0; 
+   /* BUSTOUR: */
+   statement(lab+6, 0, lev, 0, 0, rframes); /* no former frames to flush. */
+   /*        loop  swp lev   bf cf rf         */
+   definelab( lab+6 );    /* Beginn Ausstieg */
+   shframedepth = oldshframedepth;
+   isinsyncmode = 0; isinsideastartbody = 0;
+
+   b7 = tree( JBLOCKg+V, voidtype, 0, 0);
+   code(Pcode);
+   codelist->u.node = listnodes( b7, 0, 0 );
+   codelist->par = joinid;
+      /* restore ticket, gone; restore group frames, restore $ */
+   walk(0,0,0);   /*damit vorangehende stmts geflusht werden*/
+
+   branch(lab + 2);    /* vom Ausstieg zum Join-Ende springen */
+
+   if (t == ELSE) {   /* optional else part */
+      t = gettok();  /*ELSE*/
+      definelab(lab+5);
+      b8 = tree( JBLOCKh+V, voidtype, 0, 0); /* restore $ */
+      code(Pcode);
+      codelist->u.node = listnodes( b8, 0, 0 );
+      codelist->par = joinid;
+      walk(0,0,0);   /*damit vorangehende stmts geflusht werden*/
+      /* busywait: */
+      statement(lab, swp, lev,       0,      0, rframes);
+      /*                         bframes  cframes           */
+      /* if ref */
+      definelab(lab);
+      /* continue springt nach lab+1, d.h. ganz an den Anfang von join */
+      /* break springt nach lab+2, d.h. ganz ans Ende von join */
+      branch( lab + 2 ); /* falls kein continue kam: nicht nochmal versuchen */
+      walk(0,0,0);
+   }
+   else {
+      /* default-Else: wird erzeugt, wenn kein else codiert ist.
+       * Hier kommt hin, wer den Bus verpasst hat 
+       * oder wer absprang und bereits aus der Busgruppe ausgestiegen ist
+       */
+      definelab(lab+5);
+      /* Springe ans Join-Ende und versuche es nicht nochmal.*/
+      branch( lab+2 );   /* d.h. else break; */
+      walk(0,0,0);
+   }
+
+   /* Join-Ende*/
+   definelab(lab + 2);
+   walk(0,0,0);
+
+}  /* joinstmt() */
+
+
 /* xxxforstmt - xxxfor ( [expr1] ; [expr2] ; [expr3] ) statement */
 static void xxxforstmt(lab, swp, lev, rframes) struct swtch *swp; {
    Tree e1, e2, e3;
@@ -435,7 +572,7 @@ static void forkstmt(lab, swp, lev, bframes, cframes, rframes) struct swtch *swp
    }
    if (e1) {
       e1 = tree(MKGRPV, voidtype, e1, 0 );    /*CWK*/
-      code(Gen); 
+      code(Gen);
       codelist->u.node = listnodes(e1, 0, 0);
    }  /* spoolout */
    definept(0);
@@ -491,9 +628,7 @@ static void startstmt(lab, swp, lev) struct swtch *swp; {
    Tree e;
    extern void sproc(), xproc();
    extern int shframedepth; 
-   extern int shoffset;
    int oldshframedepth;
-   int oldshoffset;	
 
    t = gettok();
    if (t=='(')  {  /*Expression follows*/
@@ -816,6 +951,15 @@ void statement(loop, swp, lev, bframes, cframes, rframes) struct swtch *swp; {
       }
       break;
       }
+   case JOIN:           /*990126*/
+      if (isFORKmodul)
+        if (!isinsyncmode)
+          joinstmt(genlabel(7), swp, lev+1, rframes);   /*CWK*/
+        else
+          error("join only permitted in asynchronous mode");
+      else
+        error("join is a Fork95-specific statement, compile with -FORK");
+      break;
    case BEGINSYNC:        /* 950912 */
       if (isinsyncmode)
          error("beginsync statement is not allowed in synchronous context\n");
